/**
 * @(#)ClusterRect.java  12/12/99
 * @author Peter Au
 * All rights reserved.
 */

package vishnu.testvis.treemap.rectangle;

import vishnu.testvis.object.ClusterObj;
import java.awt.*;


public class ClusterRect extends ZRect
{
    private ClusterObj _data;
	public ClusterRect(float x1, float y1, float x2, float y2, Color c, ClusterObj o)
    {
		super(x1,y1,x2,y2,c,o.getDocsSize());
        _data = o;
	}

  public void paint(Graphics oldg){
    Graphics2D g = (Graphics2D) oldg;
    g.setColor(getColor());
    g.fill(getBounds2D());
    g.setColor(Color.white);
    g.draw(getBounds2D());
  }

  public ClusterObj getClusterObj(){return _data;}

  public String getInfo(){return _data.getInfo();}


  //------------------------------------------------------------------
  private int _xRef=0, _yRef=0;   // used in paintLabels and createTipBox

  /**
   * paint the cluster's labels on the graphical pane
   * @param oldg is the graphics to be pained
   * @param w is the max width of the graphic
   * @param h is the max heigth of the graphic
   */
  public void paintLabels(Graphics oldg, float w, float h){
  Color tipBoxColor = new Color(255,255,180);
  Graphics2D g = (Graphics2D) oldg;
  Font f = new Font("TimesRoman",Font.PLAIN,12);
  FontMetrics fm = g.getFontMetrics(f);
  int ascent = fm.getAscent();
  int height = fm.getHeight();
  int boxWidth = 4 + Math.max(fm.stringWidth(_data.getLabel1()),
                              fm.stringWidth(_data.getLabel2()) );
  int boxHeight = 0 + height * 2;
  Polygon poly;

    g.setFont(f);

    //-- setup the position of the tipbox
    poly = createTipBox(boxWidth, boxHeight, w, h);

    //-- paint the box
    g.setColor(tipBoxColor);
    g.fillPolygon(poly);
    g.setColor(Color.black);
    g.drawPolygon(poly);
    //-- paint the text
    g.drawString(_data.getLabel1(), _xRef+3F,
                _yRef + (boxHeight-(height*2F))/2F + ascent);
    g.drawString(_data.getLabel2(), _xRef+3F,
                _yRef + (boxHeight-(height*2F))/2F + ascent + height);

  }


  private Polygon createTipBox(int boxW, int boxH, float maxW, float maxH){
  int[] xs = new int[8];
  int[] ys = new int[8];
  int x=0, y=0,
    arrowH=10,
    arrowW=8,
    arrowOffset=8,
    gap=5; //-- gap is the space between the tipbox and the rectangle
  float position=0.2F;  //-- set the box position

    try{

      //-- try RHS
      x = (int) getLowerRightX() + gap;
      y = (int) (getUpperLeftY() + getHeight()*position);
      if ((x+boxW<=maxW) && (y+boxH<=maxH)){
        xs[0]=xs[6]=xs[4]=xs[3]=xs[7] = x;
        xs[1]=xs[2] = x + boxW;
        xs[5] = x - arrowH;
        ys[5]=ys[0]=ys[1]=ys[7] = y;
        ys[3]=ys[2] = y + boxH;
        ys[6] = y + arrowOffset;
        ys[4] = y + arrowOffset + arrowW;
        throw new Exception();
      }

      //-- try LHS
      x = (int) getUpperLeftX() - gap - boxW;
      if ((x>=0) && (y+boxH<=maxH)){
        xs[0]=xs[7]=xs[6] = x;
        xs[1]=xs[2]=xs[4]=xs[5] = x + boxW;
        xs[3] = x + boxW + arrowH;
        ys[7]=ys[0]=ys[1]=ys[3] = y;
        ys[6]=ys[5] = y + boxH;
        ys[2] = y + arrowOffset;
        ys[4] = y + arrowOffset + arrowW;
        throw new Exception();
      }

      //-- try top
      x = (int) (getUpperLeftX() + getWidth()*position);
      y = (int) getUpperLeftY() - boxH - gap;
      if ((y>=0) && (x+boxW<=maxW)){
        xs[0]=xs[7]=xs[6]=xs[4] = x;
        xs[1]=xs[2] = x + boxW;
        xs[5] = x + arrowOffset;
        xs[3] = x + arrowOffset + arrowW;
        ys[7]=ys[0]=ys[1] = y;
        ys[6]=ys[5]=ys[3]=ys[2] = y + boxH;
        ys[4] = y + boxH + arrowH;
        throw new Exception();
      }

      //-- try bottom
      y = (int) getLowerRightY() + gap;
      if ((y+boxH<=maxH) && (x+boxW<=maxW)){
        xs[7]=xs[0]=xs[6]=xs[2] = x;
        xs[4]=xs[5] = x + boxW;
        xs[1] = x + arrowOffset;
        xs[3] = x + arrowOffset + arrowW;
        ys[7]=ys[0]=ys[1]=ys[3]=ys[4]= y;
        ys[6]=ys[5] = y + boxH;
        ys[2] = y - arrowH;
        throw new Exception();
      }

      //-- try inside the rectangle
      x = (int) getUpperLeftX() + 5;
      y = (int) getUpperLeftY() + 5 + 5;
      xs[7]=xs[0]=xs[6]=xs[2] = x;
      xs[4]=xs[5] = x + boxW;
      xs[1] = x + 3;
      xs[3] = x + 3 + 5;
      ys[7]= ys[0]=ys[1]=ys[3]=ys[4] = y;
      ys[6]= ys[5]   = y + boxH;
      ys[2] = y - 5;

    }
    catch(Exception e){
    }

    _xRef = x;
    _yRef = y;
    return (new  Polygon(xs,ys,xs.length));
  }


}

